@@ -9,7 +9,7 @@ See the user interface documentation at https://tum-pbs.github.io/PhiFlow/Visual
 """
 from ._viewer import Viewer
 from ._matplotlib import plot_scalars
-from ._vis import view, control, show, action, plot, overlay, write_image, write_image as savefig
+from ._vis import view, control, show, close, action, plot, overlay, write_image, write_image as savefig
 
 __all__ = [key for key in globals().keys() if not key.startswith('_')]
 
